# Provides a bunch of general purpose functions for boring sys-related
# stuff
# Part of the Geeneus package
#
# Copyright 2012-2015 by Alex Holehouse, with significant contributions
# from Matt Matlock - see LICENSE for more info
# Contact at alex.holehouse@wustl.edu


def show_warning(msg):
    print "[WARNING]: " + str(msg)

def show_error(msg):
    print "[ERROR]: " + str(msg)

def show_status(msg):
    print "[STATUS]: " + str(msg)
    
